#classe Hex : Représente une cellule du plateau
    #position : coordonnées de la cellule en format (x, y)
    #owner : id du joueur auquel appartient la cellule, 0 si libre
    #pathLengthFromStart : stocke le nombre de cellules innocupées du chemin le plus court allant d'une cellule de départ jusqu'à la cellule en question
    #neighbors : cellules voisines de la cellule en question 
class Hex:
    def __init__(self, position : tuple, owner : int = 0) :
        self.position = position
        self.owner = owner
        self.pathLengthFromStart = float("inf")
        self.neighbors = []

    def print(self) :
        print (self.position, self.owner, end = ' ')


#classe Board : Représente le plateau contenant les cellules.
    #board_size : taille du tableau
    #hexes : dictionnaire stockant les cellules du tableau, la clé est un tuple de position et la valeur est un objet Hex
class Board:
    def __init__(self, board_size) :
        self.board_size : int = board_size
        self.hexes = {}
        #Ajout des 4 cellules virtuelles qui n'existent que pour l'algorithme de Dijkstra
        self.hexes[(-1,0)] = Hex((-1,0), 1)
        self.hexes[(0,-1)] = Hex((0,-1), 1)
        self.hexes[(-2,0)] = Hex((-2,0), 2)
        self.hexes[(0,-2)] = Hex((0,-2), 2)
        #Ajout de toutes les cellules du tableau
        for i in range(board_size) :
            for j in range(board_size) :
                self.hexes[(i,j)] = Hex((i,j))
        #Complétion du tableau neighbors pour chaque objet Hex
        for i in range(board_size) : 
            for j in range(board_size) :
                self.determineNeighbors(self.hexes[(i,j)])
        self.determineNeighbors(self.hexes[(-1,0)])
        self.determineNeighbors(self.hexes[(0,-1)])
        self.determineNeighbors(self.hexes[(-2,0)])
        self.determineNeighbors(self.hexes[(0,-2)])
    
    #majBoard() : met à jour les owner des cellules du board en fonction du board passé en paramètre
        #board : repésentation du plateau en tableau 2D qui contient pour chaque case l'id du joueur la possédant
    def majBoard(self, board) :
        for i in self.board_size :
            for j in self.board_size :
                self.hexes[(i,j)].owner = board[i][j]
    
    #calculHeuristic() : retourne un heuristique basé sur Dijkstra en fonction de l'état du plateau et du coup à traité
        #newBoard : repésentation du plateau en tableau 2D qui contient pour chaque case l'id du joueur la possédant
        #posCellConvoitee : position de la cellule convoitée pour le coup à évaluer
        #idJoueur : id du joueur actuel
    def calculHeuristic(self, newBoard, posCellConvoitee, idJoueur) : 
        #MAJ du plateau qui sert uniquement à Dijkstra et on ajoute le coup en cours de traitement au niveau des owner de cellules
        self.majBoard(newBoard)
        self.hexes[posCellConvoitee].owner = idJoueur

        #retour du score en fonction du joueur (longueurPlusPetitCheminDuJoueur1 - longueurPlusPetitCheminDuJoueur2)
        #pour le joueur 1, l'inverse si joueur 2
        startHexJ1 = self.hexes[(-1,0)]
        endHexJ1 = self.hexes[(0,-1)]
        startHexJ2 = self.hexes[(-2,0)]
        endHexJ2 = self.hexes[(0,-2)]
        if idJoueur == 1 :
            return (self.findShortestPathDijkstraMode(startHexJ1, endHexJ1) - self.findShortestPathDijkstraMode(startHexJ2, endHexJ2))
        else :
            return (self.findShortestPathDijkstraMode(startHexJ1, endHexJ1) - self.findShortestPathDijkstraMode(startHexJ2, endHexJ2))
    
    #findShortestPathDijkstraMode() : point d'entrée de l'algorithme de pathfing. 
    #Le but est de trouver le chemin le plus court entre startHex et endHex, en utilisant
    #cette version de Dijkstra remaniée par mes soins on itère sur toutes les cellules (vides ou nôtre)
    #pour trouver la longueur du chemin le plus court qui les sépare de startHex avant de renvoyer 
    #celle de endHex
        #startHex : départ du chemin à trouver
        #endHex : arrivé du chemin à trouver
    def findShortestPathDijkstraMode(self, startHex : Hex, endHex : Hex) :
        owner = startHex.owner
        
        #on remet à la valeure d'origine pathLentghFromStart de toutes les cellules
        self.hexes[(-1,0)].pathLengthFromStart = float('inf')
        self.hexes[(0,-1)].pathLengthFromStart = float('inf')
        self.hexes[(-2,0)].pathLengthFromStart = float('inf')
        self.hexes[(0,-2)].pathLengthFromStart = float('inf')
        for i in range(self.board_size) :
            for j in range(self.board_size) : 
                self.hexes[(i,j)].pathLengthFromStart = float('inf')
        #pathLengthFromStart de startHex doit être à 0 car de startHex à startHex, aucun coups n'est requis
        startHex.pathLengthFromStart = 0
        #parcours récursif de voisins
        self.dijkstraRec(startHex, owner)
        #renvoie de la longueur du plus court chemin qui sépare startHex et endHex
        return endHex.pathLengthFromStart    

    #dijkstraRec() : fonction récursive de parcours de voisins et de MAJ de pathLengthFromStart de findShortestPathDijkstraMode
        #currentHex : cellules dont on veut parcourir les voisins
        #owner : id du joueur pour lequel on veut trouver la longueur du plus court chemin
    def dijkstraRec(self, currentHex : Hex, owner : int) : 
        neighborsHexes = currentHex.neighbors
        #parcours des voisins de currentHex
        for currentNeighbor in neighborsHexes : 
            #filtrage des voisins qui sont soit innocupés soit occupé par le joueur dont l'id est owner
            if  (currentNeighbor.owner == 0 or currentNeighbor.owner == owner) :
                #si le voisin est déjà au joueur d'id owner, alors ça ne coûte rien de le traverser, sinon 1
                theoricNewLengthh = currentHex.pathLengthFromStart + 1 if currentNeighbor.owner == 0 else currentHex.pathLengthFromStart
                #enfin, si theoricNewLength est inférieur au pathLengthFromStart du voisin en cours de traitement, on met à 
                #jour son pathLengthFromStart et on continue récursivement sur ses voisins à lui 
                if theoricNewLengthh < currentNeighbor.pathLengthFromStart : 
                    currentNeighbor.pathLengthFromStart = theoricNewLengthh
                    self.dijkstraRec(currentNeighbor, owner)

    #determineNeighbors() : remplis le tableau de voisins de hex avec ses voisins
        #hex : cellule dont on veut remplir le tableau de voisin
    def determineNeighbors(self, hex: Hex) :
        #on récupères la position des voisins de hex
        neighborsPos = self.getNeighborsPos(hex.position)
        #on ajoute les voisins correspondant
        for value in neighborsPos :
            hex.neighbors.append(self.hexes[value])
    
    #getNeighborsPos() : retourne le tableua des position des voisins de la cellule correspondant à la position passée en paramètre
        #posTarget : position de la cellule dont on veut récupérer la position des voisins 
    def getNeighborsPos(self, posTarget: tuple):
        posNeighbors = []
        #traitement spécial pour les cellules virtuelles crées pour Dijkstra
        if posTarget == (-2, 0):
            for i in range(self.board_size):
                posNeighbors.append((0, i))
        elif posTarget == (0, -2):
            for i in range(self.board_size):
                posNeighbors.append((5, i))
        elif posTarget == (-1, 0):
            for i in range(self.board_size):
                posNeighbors.append((i, 0))
        elif posTarget == (0, -1):
            for i in range(self.board_size):
                posNeighbors.append((i, 5))
        else:
            #haut gauche
            if posTarget[0] - 1 >=0 : 
                posNeighbors.append((posTarget[0]-1, posTarget[1]))
            #haut droit
            if posTarget[0] - 1 >= 0 and posTarget[1] + 1 < self.board_size:
                    posNeighbors.append((posTarget[0] - 1, posTarget[1]+1))
            #droit
            if posTarget[1] + 1 < self.board_size:
                    posNeighbors.append((posTarget[0], posTarget[1]+1))
            #bas droit
            if posTarget[0] + 1  < self.board_size:
                    posNeighbors.append((posTarget[0] + 1, posTarget[1]))
            #bas gauche
            if posTarget[0]+1 < self.board_size and posTarget[1] - 1 >=0 :
                    posNeighbors.append((posTarget[0]+1, posTarget[1] - 1))
            #gauche
            if posTarget[1] - 1 >= 0:
                    posNeighbors.append((posTarget[0], posTarget[1]-1))

            #si la cellule est sur un bord du plateau
            if posTarget[0] == 0:
                posNeighbors.append((-2, 0))
            if posTarget[0] == self.board_size-1:
                posNeighbors.append((0, -2))
            if posTarget[1] == 0:
                posNeighbors.append((-1, 0))
            if posTarget[1] == self.board_size-1:
                posNeighbors.append((0, -1))
        return posNeighbors
        #Pour Dijkstra, il faut représenter 4 nouvelles cases fictives
        #correspondants à l'extérieur de chaque bords du plateau. Par
        #exemple, pour le haut, on aura une case appartenant aux blancs
        #qui a pour voisins toute la première ligne du plateau. Pour les
        #représenter et les différencier on utilise : (-2, 0) : haut
        #(0, -2) : bas ; (-1, 0)  : gauche ; (0, -1) : droite