import os
import pickle
import logging
import time

from rich import print
from rich.logging import RichHandler

FORMAT = "%(message)s"
logging.basicConfig(level="NOTSET", format=FORMAT, datefmt="[%X]", handlers=[RichHandler()])

# Hide Pygame welcome message
os.environ["PYGAME_HIDE_SUPPORT_PROMPT"] = "hide"
import pygame

from classes.game import Game

import pandas as pd



class Tournament:
    def __init__(self, args:  list):
        """
        Initialises a tournament with:
           * the size of the board,
           * the playing mode, i.e., "ai_vs_ai", "man_vs_ai",
           * the game counter,
           * the number of games to play.
        """
        self.args = args
        self.BOARD_SIZE = args[0]
        self.MODE = args[1]
        self.GAME_COUNT = args[2]
        self.N_GAMES = args[3]
        self.white_time = 0
        self.black_time = 0
    def single_game(self, black_starts: bool = True) -> int:
        """
        Runs a single game between two opponents.

        @return   The number of the winner, either 1 or 2, for black and white respectively.
        """
        pygame.init()
        pygame.display.set_caption("Polyline")
        start_time = time.time()
        game = Game(board_size = self.BOARD_SIZE, mode = self.MODE, black_starts = black_starts,black_strat=2)
        game.get_game_info([ self.BOARD_SIZE, self.MODE, self.GAME_COUNT ])
        while game.winner is None:
            game.play()
        self.black_time += game.logic.black_time
        self.white_time += game.logic.white_time
        return game.winner

    def championship(self):
        """
        Runs a number of games between the same two opponents.
        """
        count = 0
        for _ in range(self.N_GAMES):
            self.GAME_COUNT = _

            # First half of the tournament started by one player.
            # Remaining half started by other player (see "no pie rule")
            winner = self.single_game(black_starts = self.GAME_COUNT < self.N_GAMES / 2)
            if(winner == 1):
                count += 1
        #comparaison here
        print("the black player took " + str(self.black_time/self.N_GAMES) + "to play in average per game")
        print("They won " + str(count) + "game out of " + str(self.N_GAMES))

        print("the white player took " + str(self.white_time / self.N_GAMES) + "to play in average per game")
        print("They won " + str(self.N_GAMES - count) + "game out of " + str(self.N_GAMES)) 
        log = logging.getLogger("rich")
